using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using CrystalDecisions.Enterprise;
using CrystalDecisions.Enterprise.Desktop;
using BOE.BOECommonInfo;

namespace BOE.BOEPublishFiles
{
  public class BOEPublishFiles
  {
    private BOECommon _common = null;
    private string _errMsg = string.Empty;

    #region Class_Methods
    /// <summary>
    /// Default constructor.  Included in case we need to serialize the object.
    /// DO NOT USE!!!
    /// </summary>
    public BOEPublishFiles()
    {
      //Default constructor - DO NOT USE!  
    }

    /// <summary>
    /// Create a BOECommon to connect to CMS and initialize fields.
    /// </summary>
    /// <param name="userName">User name for logging into CMS</param>
    /// <param name="password">Password for loggin into CMS</param>
    /// <param name="cms">CMS Name</param>
    public BOEPublishFiles(string userName, string password, string cms)
    {
      _common = new BOECommon(userName, password, cms);
      _errMsg = _common.ErrMsg;
    }

     ~BOEPublishFiles()
    {
      Dispose();
    }

    /// <summary>
    /// Logg off CMS and free up resources used by this object
    /// </summary>
    public void Dispose()
    {
      if (_common != null)
        _common.Dispose();
    }

    public string ErrMsg
    {
      get { return _errMsg; }
    }

    public BOECommon CommonInfo
    {
      get { return _common; }
    }

    #endregion Class_Methods

    #region Public
    /// <summary>
    /// Publish a file to BusinessObjects.  Title is based on the file name.
    /// </summary>
    /// <param name="fileName">The file to be published</param>
    /// <param name="folderPath">The path in BO to which the file will be published</param>
    /// <param name="title">The title for the file.</param>
    /// <returns></returns>
    public bool PublishFile(string fileName, string folderPath, string title)
    {
      bool result = true;
      try
      {
        if (!System.IO.File.Exists(fileName))
        {
          throw (new Exception(fileName + " does not exist."));
        }
        Int32 folderId = _common.GetFolderIDFromPath(folderPath, 0);
        if (folderId < 0)
        {
          throw (new Exception(folderPath + " does not exist in BusinessObjects."));
        }
        string tmp = title;
        int i = 1;
        while (_common.GetIDFromName(tmp, folderId) > 0)
        {
          i++;
          tmp = title + '(' + i.ToString() + ')';
        }
        title = tmp;
        FileType fileType = getFileType(fileName);
        if (fileType != FileType.Unknown)
        {
          switch (fileType)
          {
            case FileType.Excel:
              result = publishXLS(fileName, title, folderId);
              break;
            case FileType.PDF:
              result = publishPDF(fileName, title, folderId);
              break;
            case FileType.PowerPoint:
              result = publishPPT(fileName, title, folderId);
              break;
            case FileType.RTF:
              result = publishRTF(fileName, title, folderId);
              break;
            case FileType.Text:
              result = publishTXT(fileName, title, folderId);
              break;
            case FileType.Word:
              result = publishDOC(fileName, title, folderId);
              break;
            default:
              result = false;
              _errMsg = "Unknown File Type";
              break;
          }
        }
      }
      catch (Exception ex)
      {
        result = false;
        _errMsg = "Unable to publish file. Error=" + ex.Message;
      }

      return result;
    }

    /// <summary>
    /// Publish a file to BusinessObjects.  Title is based on the file name.
    /// </summary>
    /// <param name="fileName">The file to be published</param>
    /// <param name="folderPath">The path in BO to which the file will be published</param>
    public bool PublishFile(string fileName, string folderPath)
    {
      string title = folderPath;
      int i = 0;
      i = folderPath.LastIndexOf('\\');
      if (i >= 0)
      {
        title = folderPath.Substring(i + 1);
      }
      i = fileName.LastIndexOf('\\');
      string tmp = fileName.Substring(i + 1);
      i = tmp.LastIndexOf('.');
      title = title + " - " + tmp.Substring(0, i);
      bool result= PublishFile(fileName, folderPath, title);
      return result;
    }

    #endregion Public

    #region Private
    /// <summary>
    /// Get the type of file to publish based on the file extension.
    /// The default file type is Text.
    /// </summary>
    /// <param name="fileName"></param>
    /// <returns></returns>
    private FileType getFileType(string fileName)
    {
      int dotAt = fileName.LastIndexOf('.');
      string ext = string.Empty;
      FileType result = FileType.Unknown;
      if (dotAt > 0)
      { 
        ext = fileName.Substring(dotAt + 1, 3).ToUpper();
      }
      switch (ext)
      {
        case "XLS":
          result = FileType.Excel;
          break;
        case "PDF":
          result = FileType.PDF;
          break;
        case "PPT":
          result = FileType.PowerPoint;
          break;
        case "RTF":
          result = FileType.RTF;
          break;
        case "DOC":
          result = FileType.Word;
          break;
        default:
          result = FileType.Text;
          break;
      }
      return result;
    }
  
    /// <summary>
    /// Publish a Microsoft Excel file.
    /// </summary>
    /// <param name="fileName">Name and full path of the file to be published</param>
    /// <param name="title">Title of the file to be published</param>
    /// <param name="folderId">ID of the folder to which the file will be published</param>
    /// <returns></returns>
    private bool publishXLS(string fileName, string title, Int32 folderId)
    {
      bool result = true;
      try
      {
        InfoObjects xlsObjs = _common.BOEInfoStore.NewInfoObjectCollection();
        Excel xls = (Excel)xlsObjs.AddEx("Excel");
        xls.Title = title;
        xls.Files.Add(fileName);
        xls.ParentID = folderId;
        _common.BOEInfoStore.Commit(xlsObjs);
      }
      catch (Exception ex)
      {
        result = false;
        _errMsg = "Unable to save " + fileName + " as an Excel document.  Err=" + ex.Message;
      }
      return result;
    }

    /// <summary>
    /// Publish an Adobe Acrobat file.
    /// </summary>
    /// <param name="fileName">Name and full path of the file to be published</param>
    /// <param name="title">Title of the file to be published</param>
    /// <param name="folderId">ID of the folder to which the file will be published</param>
    /// <returns></returns>
    private bool publishPDF(string fileName, string title, Int32 folderId)
    {
      bool result = true;
      try
      {
        InfoObjects pdfObjs = _common.BOEInfoStore.NewInfoObjectCollection();
        Pdf pdf = (Pdf)pdfObjs.AddEx("Pdf");
        pdf.Title = title;
        pdf.Files.Add(fileName);
        pdf.ParentID = folderId;
        _common.BOEInfoStore.Commit(pdfObjs);
      }
      catch (Exception ex)
      {
        result = false;
        _errMsg = "Unable to save " + fileName + " as a PDF document.  Err=" + ex.Message;
      }
      return result;
    }

    /// <summary>
    /// Publish a Microsoft Powerpoint file.
    /// </summary>
    /// <param name="fileName">Name and full path of the file to be published</param>
    /// <param name="title">Title of the file to be published</param>
    /// <param name="folderId">ID of the folder to which the file will be published</param>
    /// <returns></returns>
    private bool publishPPT(string fileName, string title, Int32 folderId)
    {
      bool result = true;
      try
      {
        InfoObjects pptObjs = _common.BOEInfoStore.NewInfoObjectCollection();
        Powerpoint ppt = (Powerpoint)pptObjs.AddEx("Powerpoint");
        ppt.Title = title;
        ppt.Files.Add(fileName);
        ppt.ParentID = folderId;
        _common.BOEInfoStore.Commit(pptObjs);
      }
      catch (Exception ex)
      {
        result = false;
        _errMsg = "Unable to save " + fileName + " as a Powerpoint document.  Err=" + ex.Message;
      }
      return result;
    }

    /// <summary>
    /// Publish an RTF file.
    /// </summary>
    /// <param name="fileName">Name and full path of the file to be published</param>
    /// <param name="title">Title of the file to be published</param>
    /// <param name="folderId">ID of the folder to which the file will be published</param>
    /// <returns></returns>
    private bool publishRTF(string fileName, string title, Int32 folderId)
    {
      bool result = true;
      try
      {
        InfoObjects rtfObjs = _common.BOEInfoStore.NewInfoObjectCollection();
        Rtf rtf = (Rtf)rtfObjs.AddEx("Rtf");
        rtf.Title = title;
        rtf.Files.Add(fileName);
        rtf.ParentID = folderId;
        _common.BOEInfoStore.Commit(rtfObjs);
      }
      catch (Exception ex)
      {
        result = false;
        _errMsg = "Unable to save " + fileName + " as an Rtf document.  Err=" + ex.Message;
      }
      return result;
    }

    /// <summary>
    /// Publish a Text file.
    /// </summary>
    /// <param name="fileName">Name and full path of the file to be published</param>
    /// <param name="title">Title of the file to be published</param>
    /// <param name="folderId">ID of the folder to which the file will be published</param>
    /// <returns></returns>
    private bool publishTXT(string fileName, string title, Int32 folderId)
    {
      bool result = true;
      try
      {
        InfoObjects txtObjs = _common.BOEInfoStore.NewInfoObjectCollection();
        Txt txt = (Txt)txtObjs.AddEx("Txt");
        txt.Title = title;
        txt.Files.Add(fileName);
        txt.ParentID = folderId;
        _common.BOEInfoStore.Commit(txtObjs);
      }
      catch (Exception ex)
      {
        result = false;
        _errMsg = "Unable to save " + fileName + " as a Txt document.  Err=" + ex.Message;
      }
      return result;
    }

    /// <summary>
    /// Publish a Microsoft Word file.
    /// </summary>
    /// <param name="fileName">Name and full path of the file to be published</param>
    /// <param name="title">Title of the file to be published</param>
    /// <param name="folderId">ID of the folder to which the file will be published</param>
    /// <returns></returns>
    private bool publishDOC(string fileName, string title, Int32 folderId)
    {
      bool result = true;
      try
      {
        InfoObjects docObjs = _common.BOEInfoStore.NewInfoObjectCollection();
        Word doc = (Word)docObjs.AddEx("Word");
        doc.Title = title;
        doc.Files.Add(fileName);
        doc.ParentID = folderId;
        _common.BOEInfoStore.Commit(docObjs);
      }
      catch (Exception ex)
      {
        result = false;
        _errMsg = "Unable to save " + fileName + " as a Word document.  Err=" + ex.Message;
      }
      return result;
    }
    #endregion Private
  }
}
